#! /usr/bin/env python
#
# Public domain
import os
import stat
import sre
import time

PKGROOT="/var/db/pkg"
from freebsdports import PORTS

def loadfromfs():
    "Vraci seznam nainstalovanych baliku z filesystemu"
    list=[]
    for fn in os.listdir(PKGROOT):
       s=os.stat(PKGROOT+"/"+fn)
       if stat.S_ISDIR(s[stat.ST_MODE]):
           list.append(parsepkgname(fn))
    return list

def getorigin(directory):
    "Vrati origin baliku v adresari"
    ORIGINSTR=r"^@comment ORIGIN:(\S+)"
    prog=sre.compile(ORIGINSTR,sre.MULTILINE)
    f=file(directory+'/+CONTENTS','r')
    matches=sre.findall(prog,f.read())
    f.close()
    if len(matches)==1:
        return PORTS+'/'+matches[0]
    else:
        return None

def loadoriginsfromfs():
    "Vraci hastabulku originu:instdatumu (mail/exim) z filesystemu"
    list={}
    for fn in os.listdir(PKGROOT):
       s=os.stat(PKGROOT+"/"+fn)
       if stat.S_ISDIR(s[stat.ST_MODE]):
           org=getorigin(PKGROOT+'/'+fn)
           if org:
              s=os.stat(PKGROOT+'/'+fn+'/+COMMENT')
              t=time.localtime(s[stat.ST_MTIME])
              list[org]="%4d-%02d-%02d" % (t[0],t[1],t[2])
    return list
    
def loadoriginsversions():
    "Vraci hastabulku originu:inst verze z filesystemu"
    list={}
    for fn in os.listdir(PKGROOT):
       s=os.stat(PKGROOT+"/"+fn)
       if stat.S_ISDIR(s[stat.ST_MODE]):
           org=getorigin(PKGROOT+'/'+fn)
           if org:
              list[org]=makeversion(parsepkgname(fn))
    return list

#parse directory names like gconf-editor-2.4.0_5,1
def parsepkgname(name):
    PKGSTR=r"(.+)-([0-9a-zA-Z]+(\.[0-9a-zA-Z]+)*)(_\d+)?(,\d+)?"
    PKGSTR=r"(.+)-(([\.0-9a-zA-Z]+)+)(_\d+)?(,\d+)?"
    prog=sre.compile(PKGSTR)
    res=prog.search(name)
    if not res:
        print "can not parse package name:",name
        return None
    n=res.group(1)
    v=res.group(2)
    if res.group(4):
        r=res.group(4)[1:]
    else:
        r=None
    if res.group(5):
        e=res.group(5)[1:]
    else:
        e=None
    return (n,v,r,e)

def makeversion(parsed):
     version=parsed[1]
     if parsed[2]: version+='_'+parsed[2]
     if parsed[3]: version+=','+parsed[3]
     return version    

if __name__ == "__main__":
   print loadfromfs()
   print loadoriginsfromfs()
