#! /usr/bin/env python
# Loads available packages from INDEX into into database
try:
  import psyco
  psyco.profile()
except Exception:
  pass

DATABASE='test'

# python load.py [-n] [ filename ...]
# command line arguments
#   -n 		do not update package description
#   filename    import this filename (Debian available file format)
#               if no filename use apt-cache dumpavail

import os
import pg
import types
import sys
from mx.DateTime import *
from stat import *
import gzip
import getopt
from indexer import INDEX
import bsdpkg

#Datum pro released polozku v balicku
USEDATE=now()
USEDATESTR=None

class pkg:
   dbinternal={'id':None,'alive':None,'packageid':None,'install':None}
   
   def __init__(self):
      self.clear()

   def clear(self):
      self.package=None
      self.pkgdir=None
      self.section=None
      self.descr=None
      self.maintainer=None
      self.version=None
      self.bdepends=None
      self.rdepends=None
      self.www=None
      self.released=None
      self.installed=None
   
   def updatefromdb(self,res1,res2):
       res1.setdefault(None)
       res2.setdefault(None)
       for list in ( res1.keys() , res2.keys() ):
         for field in list:
	    if field is None: continue
	    if pkg.dbinternal.has_key(field): continue
	    if hasattr(self,field):
	       self.__dict__[field]=res1.get(field) or res2.get(field)
	       #print "updated value",field,self.__dict__[field]
	    else:
	       print "pkg object has no atr",field

   #save object into database
   def savetodb(self,c,userealdates):
	#have we old object in database?
	q=c.query('SELECT * FROM packages WHERE pkgdir=\''+self.pkgdir+'\'')
	if q.ntuples()==0:
	   #create a new database record
	   query="INSERT INTO packages(package,pkgdir,section,descr,maintainer,bdepends,rdepends,www) VALUES('"+self.package+"','"+self.pkgdir+"',"+quote(self.section)+","+quote(self.descr)+","+quote(self.maintainer)+","+quote(self.bdepends)+","+quote(self.rdepends)+","+quote(self.www)+")"
	   print "Inserting NEW PACKAGE",self.pkgdir,"to database."
	   try:
	      q=c.query(query)
	   except pg.error:
	      print "ERROR!",query
	      return
	   q=c.query('SELECT * FROM packages WHERE pkgdir=\''+self.pkgdir+'\'')
           res=q.dictresult()[0]
        else:   
            res=q.dictresult()[0]
            #check if main record needs to be updated
            changed=''
            if res['section'] != self.section: changed+=' section'
            if res['descr'] != self.descr: changed+=' descr'
            if res['package'] != self.package: changed+= ' package'
            if res['maintainer'] != self.maintainer: changed+=' maintainer'
            if res['rdepends'] != self.rdepends: changed+=' rdeps'
            if res['bdepends'] != self.bdepends: changed+=' bdeps'
            if res['www'] != self.www: changed+=' www'
            if len(changed)>0:    
                  query='UPDATE packages SET section='+quote(self.section)+",descr="+quote(self.descr)+",package="+quote(self.package)+",maintainer="+quote(self.maintainer)+",rdepends="+quote(self.rdepends)+",bdepends="+quote(self.bdepends)+",www="+quote(self.www)+" WHERE pkgdir='"+self.pkgdir+"'"
                  print "Updating",self.pkgdir,' New:'+changed
                  c.query(query)

        if userealdates and self.released is None:
             self.released=str(localtime(os.stat(self.pkgdir+'/Makefile')[ST_MTIME]))
	#check for version record
	q=c.query('SELECT * FROM versions WHERE packageid='+str(res['id'])+" AND  version="+quote(self.version))
	if q.ntuples()==0:
	   #create a new version record
	   print "Inserting new version",self.version,"of package",self.pkgdir,"to database."
	   query="INSERT INTO versions(released,installed,packageid,version) VALUES('"+(self.released or USEDATESTR)+"',"+quote(self.installed)+","+str(res['id'])+","+quote(self.version)+")"
	   try:
	      q=c.query(query)
	   except pg.error:
	      print "ERROR!",query
	else:
	   #update OLD version record
	   res=q.dictresult()[0]
           if userealdates:
               #update released date
               if self.released is None or \
                 not self.released.startswith(res['released']):
                   print 'me',self.released,'db',res['released']
                   print "Updating released date for version",self.version,"of",self.pkgdir
                   query="UPDATE versions SET released='"+(self.released)+"' WHERE version='"+res['version']+"' AND packageid="+str(res['packageid'])
                   try:
                      q=c.query(query)
                   except pg.error:
                      print query
           #update installed date
	   if self.installed:
	      if self.installed != res['installed']:
	       print "Updating installed date for version",self.version,"of",self.package
	       query="UPDATE versions SET installed='"+(self.installed)+"' WHERE version='"+res['version']+"' AND packageid="+str(res['packageid'])
	       try:
		  q=c.query(query)
	       except pg.error:
		  print query
   
   def load(self,file):
     "loads object values from Freebsd INDEX file format"
     entry=file.readline()
     if entry=='': 
          return
     entry=entry.strip()
     if entry=='': return # end of file
     spl=entry.split('|')
     if len(spl)!=10:
         print "Bad index entry:",entry
         return
     for i in range(0,10):
         if spl[i]=='':
             spl[i]=None    
     pkg,pdir,root,comment,descrfile,maint,cat,bdep,rdep,www=spl
     #parse package name 
     parsed=bsdpkg.parsepkgname(pkg)
     self.package=parsed[0]
     self.pkgdir=pdir
     self.version=bsdpkg.makeversion(parsed)
     self.descr=comment     
     self.maintainer=maint
     self.section=cat
     self.bdepends=bdep
     self.rdepends=rdep
     self.www=www
 
def quote(stri):
   if stri is None: return "NULL"
   if type(stri)==types.IntType:
       return str(stri)
   return "'"+stri.replace("\\","\\\\").replace("'","\\'")+"'"

#load multiple versions of package from database
def db_loadpkg(c,name,wantversion=None):
    q=c.query('SELECT * FROM packages WHERE pkgdir=\''+name+'\'')
    if q.ntuples()==0:
       return None
    res=q.dictresult()[0]

    query='SELECT * FROM versions WHERE packageid='+str(res['id'])
    if wantversion: query+=" AND version='"+wantversion+"'"
    query+=' ORDER BY released DESC'
    q=c.query(query)
    if q.ntuples()==0:
       return None
    if wantversion:
       p=pkg()
       p.updatefromdb(res,q.dictresult()[0])
       return p
    res2=q.dictresult()
    grandresult=[]
    for version in res2:
       p=pkg()
       p.updatefromdb(res,version)
       grandresult.append(p)
    return grandresult

#loads all entries from FreeBSD index file to database
def loadfile(f,c,usemakefiledates=True):
    global USEDATESTR
    USEDATESTR=str(USEDATE)
    c.query('BEGIN TRANSACTION');
    entry=pkg()
    while True:
       entry.clear()
       entry.load(f)
       if entry.package is None: break
       if entry.version is None: continue
       entry.savetodb(c,usemakefiledates)
    c.query('END')
#main   

if __name__ == '__main__':
    c=pg.connect(DATABASE)
    c.query('SET DateStyle=ISO');
    userealdates=True
    anyfile=False
    options, args = getopt.getopt(sys.argv[1:],"nh?")
    for opt in options: 
            if opt[0]=='-n':
		  userealdates=False
            elif opt[0]=='-h' or opt[0]=='-?':
                print "python",sys.argv[0]," [ options ]"
                print "This program loads /usr/ports/INDEX into PgSQL database."
                print "Options:"
                print "  -n       Use current date not Makefile date as released date"
                print "               Program runs much faster with it"
                sys.exit(0)
                                                    
    for fname in args:
	   if fname[-3:]=='.gz':
	      f=gzip.open(fname,'r',8192)
	   else:
	      f=open(fname,'r',8192)
	   USEDATE=localtime(os.stat(fname)[ST_MTIME])
	   anyfile=True
           loadfile(f,c,userealdates)
	   f.close()
    if anyfile is False:
           #load default INDEX
	   f=open(INDEX,'r',8192)
	   USEDATE=localtime(os.stat(INDEX)[ST_MTIME])
           loadfile(f,c,userealdates)
	   f.close()
    c.close()
